@@ -7,26 +7,28 @@ from minio import Minio
 from mlops.ProjectFile import ProjectFile
 from mlops.utils.logger import logger, LOG_FILE
 from git import Repo
+from torch.cuda import is_available
+from io import BytesIO
+from ast import literal_eval
 
 
 class Experiment:
 
-    def __init__(self, config_path: str = 'config.cfg', project_path: str = '.', use_localhost: bool = False,
+    def __init__(self, script, config_path, project_path: str = '.',
                  verbose: bool = True, ignore_git_check: bool = False):
         """
         The Experiment class is the interface through which all projects should be run.
-
+        :param script: path to script to run
         :param config_path: string path to configuration file
         :param project_path: string path to project directory
-        :param use_localhost: bool to indicate whether to use the local addresses in the config file
         :param verbose: verbosity
         """
+        self.script = script
         self.config = None
         self.artifact_path = None
         self.experiment_name = None
         self.experiment_id = None
         self.config_path = config_path
-        self.use_localhost = use_localhost
         self.project_path = project_path
         self.verbose = verbose
 
@@ -34,14 +36,15 @@ class Experiment:
             logger.warn('DEBUG ONLY - ignoring git checks due to test run detected')
 
         elif ignore_git_check is True:
-            logger.warn('DEBUG ONLY - ignoring git checks, manually disabled. Ensure this run is not for any experiments '
-                        'intended for production use')
-
+            logger.warn(
+                'DEBUG ONLY - ignoring git checks, manually disabled. Ensure this run is not for any experiments '
+                'intended for production use')
         else:
             self.check_dirty()
 
         self.check_environment_variables()
         self.config_setup()
+        self.use_gpu = self.config.getboolean('system', 'USE_GPU')
         self.env_setup()
         self.build_project_file()
         self.init_experiment()
@@ -104,12 +107,8 @@ class Experiment:
         Stores the variables required for running mlflow projects with docker in the environment
         :return:
         """
-        if self.use_localhost:
-            os.environ['MLFLOW_TRACKING_URI'] = self.config['server']['LOCAL_REMOTE_SERVER_URI']
-            os.environ['MLFLOW_S3_ENDPOINT_URL'] = self.config['server']['LOCAL_MLFLOW_S3_ENDPOINT_URL']
-        else:
-            os.environ['MLFLOW_TRACKING_URI'] = self.config['server']['REMOTE_SERVER_URI']
-            os.environ['MLFLOW_S3_ENDPOINT_URL'] = self.config['server']['MLFLOW_S3_ENDPOINT_URL']
+        os.environ['MLFLOW_TRACKING_URI'] = self.config['server']['MLFLOW_TRACKING_URI']
+        os.environ['MLFLOW_S3_ENDPOINT_URL'] = self.config['server']['MLFLOW_S3_ENDPOINT_URL']
 
     def init_experiment(self):
         """
@@ -158,10 +157,7 @@ class Experiment:
 
         :return:
         """
-        if self.use_localhost:
-            self.uri_formatted = self.config['server']['LOCAL_MLFLOW_S3_ENDPOINT_URL'].replace("http://", "")
-        else:
-            self.uri_formatted = self.config['server']['MLFLOW_S3_ENDPOINT_URL'].replace("http://", "")
+        self.uri_formatted = self.config['server']['MLFLOW_S3_ENDPOINT_URL'].replace("http://", "")
 
         self.minio_cred = {'user': os.getenv('AWS_ACCESS_KEY_ID'),
                            'password': os.getenv('AWS_SECRET_ACCESS_KEY')}
@@ -194,16 +190,31 @@ class Experiment:
             build_args = {'http_proxy': os.getenv('http_proxy'),
                           'https_proxy': os.getenv('https_proxy')}
 
-        client = docker.from_env()
         logger.info('Running docker build with: {0}'.format({'path': path if path else self.project_path,
                                                              'tag': self.experiment_name,
                                                              'buildargs': build_args,
                                                              'rm': ''}))
 
-        client.images.build(path=self.project_path,
-                            tag=self.experiment_name,
-                            buildargs=build_args,
-                            rm=True)
+        try:
+            docker_base_url = 'unix://var/run/docker.sock'
+            cli = docker.APIClient(base_url=docker_base_url)
+            valid_cli = True
+        except:
+            logger.warn(f'Low level Docker SDK not available on non-unix systems, the build will continue but will not output any logs')
+            valid_cli = False
+
+        if valid_cli:
+            for line in cli.build(path=self.project_path, tag=self.experiment_name, use_config_proxy=True):
+                block = line.decode('utf-8').splitlines()
+                block_dict = literal_eval(block[0])
+                if 'stream' in block_dict.keys():
+                    print(str(block_dict['stream']), end='')
+        else:
+            client = docker.from_env()
+            client.images.build(path=self.project_path,
+                                tag=self.experiment_name,
+                                buildargs=build_args,
+                                rm=True)
 
         logger.info('Built project image: ' + self.experiment_name + ':latest')
 
@@ -214,7 +225,7 @@ class Experiment:
         :return:
         """
         logger.info('Building project file')
-        projectfile = ProjectFile(self.config, path=self.project_path, use_localhost=self.use_localhost)
+        projectfile = ProjectFile(self.config, self.config_path, self.script, path=self.project_path)
         projectfile.generate_yaml()
 
     def run(self, **kwargs):
@@ -232,7 +243,10 @@ class Experiment:
                                'rm': '',
                                }
 
-        if not self.use_localhost:
+        # if not self.use_localhost:
+        if self.use_gpu and not is_available():
+            logger.warn('requested GPU resource but none available - using CPU')
+        elif self.use_gpu and is_available():
             gpu_params = {'gpus': 'all',
                           'runtime': 'nvidia'}
             logger.info('Adding docker args: {0}'.format(gpu_params))
@@ -258,7 +272,7 @@ class Experiment:
 
         mlflow.run(uri=self.project_path,
                    experiment_id=self.experiment_id,
-                   use_conda=False,
+                   env_manager='local',
                    **kwargs)
 
         mlflow.log_artifact(LOG_FILE)
